 10 3 4 21 20 29 42 61 0 0
3,2,1208.5,39.4575,118.77388
2,1,-999
1.25,0.5,0.12,0.12,1000
Fallon AgriMet Station - Daily
Longitude of timezone needed for hourly data)and Output flags:
120
0,0,2,1,1,1,0,1,0
0
0
3
0
0
0
0
0
0
0
0
0
0
0
0
0
0
0
0
 0 
11,0.16,0,1,2,1,0,90
abaaaaaaa, <--delimiter code.  The next line contains 24hr, daytime, &nighttime rs for alfalfa and for grass
45,30,200,70,50,200
NEW DEFINITIONS are used in this definition file beginning with version 4.1 in Nov. 2014
This definition file is named:
"Y:\morton\RefET\FALN_Agrimet_daily.def"
This definition file is used to read data file:
"Y:\morton\RefET\FALN_Agrimet_daily_raw_2015.csv"

This definition file was created by and is used by program REF-ET Windows Ver 4.1
NOTE:  It is important that you do not change the order or structure
       of this definition file.  Otherwise, REF-ET may have difficulty
       reading it the next time it is utilized.
